@@ -1,8 +1,10 @@
 import re
 
+from rope.base.utils.datastructures import OrderedSet
 from rope.base import ast, codeanalyze
 from rope.base.change import ChangeSet, ChangeContents
 from rope.base.exceptions import RefactoringError
+from rope.base.utils import pycompat
 from rope.refactor import (sourceutils, similarfinder,
                            patchedast, suites, usefunction)
 
@@ -12,7 +14,7 @@ from rope.refactor import (sourceutils, similarfinder,
 #
 # _ExtractInfo: holds information about the refactoring; it is passed
 # to the parts that need to have information about the refactoring
-# 
+#
 # _ExtractCollector: merely saves all of the information necessary for
 # performing the refactoring.
 #
@@ -36,7 +38,6 @@ class _ExtractRefactoring(object):
     def __init__(self, project, resource, start_offset, end_offset,
                  variable=False):
         self.project = project
-        self.pycore = project.pycore
         self.resource = resource
         self.start_offset = self._fix_start(resource.read(), start_offset)
         self.end_offset = self._fix_end(resource.read(), end_offset)
@@ -95,9 +96,9 @@ class _ExtractInfo(object):
 
     def __init__(self, project, resource, start, end, new_name,
                  variable, similar, make_global):
-        self.pycore = project.pycore
+        self.project = project
         self.resource = resource
-        self.pymodule = self.pycore.resource_to_pyobject(resource)
+        self.pymodule = project.get_pymodule(resource)
         self.global_scope = self.pymodule.get_scope()
         self.source = self.pymodule.source_code
         self.lines = self.pymodule.lines
@@ -153,8 +154,8 @@ class _ExtractInfo(object):
     @property
     def one_line(self):
         return self.region != self.lines_region and \
-               (self.logical_lines.logical_line_in(self.region_lines[0]) ==
-                self.logical_lines.logical_line_in(self.region_lines[1]))
+            (self.logical_lines.logical_line_in(self.region_lines[0]) ==
+             self.logical_lines.logical_line_in(self.region_lines[1]))
 
     @property
     def global_(self):
@@ -163,7 +164,7 @@ class _ExtractInfo(object):
     @property
     def method(self):
         return self.scope.parent is not None and \
-               self.scope.parent.get_kind() == 'Class'
+            self.scope.parent.get_kind() == 'Class'
 
     @property
     def indents(self):
@@ -182,6 +183,7 @@ class _ExtractInfo(object):
         return self.source[self.region[0]:self.region[1]]
 
     _returned = None
+
     @property
     def returned(self):
         """Does the extracted piece contain return statement"""
@@ -273,7 +275,8 @@ class _ExtractPerformer(object):
                 if self.info.variable:
                     return [self.info.scope_region]
                 else:
-                    return [self.info._get_scope_region(self.info.scope.parent)]
+                    return [self.info._get_scope_region(
+                        self.info.scope.parent)]
         else:
             return [self.info.region]
 
@@ -391,8 +394,9 @@ class _ExceptionalConditionChecker(object):
                                    'contain complete statements.')
 
     def _is_region_on_a_word(self, info):
-        if info.region[0] > 0 and self._is_on_a_word(info, info.region[0] - 1) or \
-           self._is_on_a_word(info, info.region[1] - 1):
+        if info.region[0] > 0 and \
+                self._is_on_a_word(info, info.region[0] - 1) or \
+                self._is_on_a_word(info, info.region[1] - 1):
             return True
 
     def _is_on_a_word(self, info, offset):
@@ -436,7 +440,7 @@ class _ExtractMethodParts(object):
         return result
 
     def _find_temps(self):
-        return usefunction.find_temps(self.info.pycore.project,
+        return usefunction.find_temps(self.info.project,
                                       self._get_body())
 
     def get_checks(self):
@@ -468,7 +472,7 @@ class _ExtractMethodParts(object):
             result.append('@staticmethod\n')
         result.append('def %s:\n' % self._get_function_signature(args))
         unindented_body = self._get_unindented_function_body(returns)
-        indents = sourceutils.get_indent(self.info.pycore)
+        indents = sourceutils.get_indent(self.info.project)
         function_body = sourceutils.indent_lines(unindented_body, indents)
         result.append(function_body)
         definition = ''.join(result)
@@ -487,11 +491,11 @@ class _ExtractMethodParts(object):
                 args.remove(self_name)
             args.insert(0, self_name)
         return prefix + self.info.new_name + \
-               '(%s)' % self._get_comma_form(args)
+            '(%s)' % self._get_comma_form(args)
 
     def _extracting_method(self):
         return self.info.method and not self.info.make_global and \
-               _get_function_kind(self.info.scope) == 'method'
+            _get_function_kind(self.info.scope) == 'method'
 
     def _get_self_name(self):
         param_names = self.info.scope.pyobject.get_param_names()
@@ -503,7 +507,7 @@ class _ExtractMethodParts(object):
         if self.info.method and not self.info.make_global:
             if _get_function_kind(self.info.scope) == 'method':
                 self_name = self._get_self_name()
-                if  self_name in args:
+                if self_name in args:
                     args.remove(self_name)
                 prefix = self_name + '.'
             else:
@@ -557,7 +561,7 @@ class _ExtractMethodParts(object):
         if self.info.one_line or self.info.returned:
             return []
         written = self.info_collector.written | \
-                  self.info_collector.maybe_written
+            self.info_collector.maybe_written
         return list(written & self.info_collector.postread)
 
     def _get_unindented_function_body(self, returns):
@@ -577,7 +581,7 @@ class _ExtractVariableParts(object):
 
     def get_definition(self):
         result = self.info.new_name + ' = ' + \
-                 _join_lines(self.info.extracted) + '\n'
+            _join_lines(self.info.extracted) + '\n'
         return result
 
     def get_body_pattern(self):
@@ -596,19 +600,20 @@ class _FunctionInformationCollector(object):
         self.start = start
         self.end = end
         self.is_global = is_global
-        self.prewritten = set()
-        self.maybe_written = set()
-        self.written = set()
-        self.read = set()
-        self.postread = set()
-        self.postwritten = set()
+        self.prewritten = OrderedSet()
+        self.maybe_written = OrderedSet()
+        self.written = OrderedSet()
+        self.read = OrderedSet()
+        self.postread = OrderedSet()
+        self.postwritten = OrderedSet()
         self.host_function = True
         self.conditional = False
 
     def _read_variable(self, name, lineno):
         if self.start <= lineno <= self.end:
             if name not in self.written:
-                self.read.add(name)
+                if not self.conditional or name not in self.maybe_written:
+                    self.read.add(name)
         if self.end < lineno:
             if name not in self.postwritten:
                 self.postread.add(name)
@@ -668,17 +673,27 @@ class _FunctionInformationCollector(object):
         self._handle_conditional_node(node)
 
     def _For(self, node):
-        self._handle_conditional_node(node)
+        self.conditional = True
+        try:
+            # iter has to be checked before the target variables
+            ast.walk(node.iter, self)
+            ast.walk(node.target, self)
 
+            for child in node.body:
+                ast.walk(child, self)
+            for child in node.orelse:
+                ast.walk(child, self)
+        finally:
+            self.conditional = False
 
 
 def _get_argnames(arguments):
-    result = [node.arg for node in arguments.args
-              if isinstance(node, ast.arg)]
+    result = [pycompat.get_ast_arg_arg(node) for node in arguments.args
+              if isinstance(node, pycompat.ast_arg_type)]
     if arguments.vararg:
-        result.append(arguments.vararg)
+        result.append(pycompat.get_ast_arg_arg(arguments.vararg))
     if arguments.kwarg:
-        result.append(arguments.kwarg)
+        result.append(pycompat.get_ast_arg_arg(arguments.kwarg))
     return result
 
 
@@ -708,7 +723,7 @@ class _VariableReadsAndWritesFinder(object):
     def find_reads_and_writes(code):
         if code.strip() == '':
             return set(), set()
-        if isinstance(code, str):
+        if isinstance(code, unicode):
             code = code.encode('utf-8')
         node = _parse_text(code)
         visitor = _VariableReadsAndWritesFinder()
@@ -743,7 +758,11 @@ class _UnmatchedBreakOrContinueFinder(object):
             ast.walk(child, self)
         self.loop_count -= 1
         if node.orelse:
-            ast.walk(node.orelse, self)
+            if isinstance(node.orelse,(list,tuple)):
+                for node_ in node.orelse:
+                    ast.walk(node_, self)
+            else:
+                ast.walk(node.orelse, self)
 
     def _Break(self, node):
         self.check_loop()
@@ -770,6 +789,7 @@ class _UnmatchedBreakOrContinueFinder(object):
         ast.walk(node, visitor)
         return visitor.error
 
+
 def _get_function_kind(scope):
     return scope.pyobject.get_kind()
 
@@ -779,6 +799,7 @@ def _parse_text(body):
     node = ast.parse(body)
     return node
 
+
 def _join_lines(code):
     lines = []
     for line in code.splitlines():
